@@ -1,27 +1,28 @@
 use std::marker::PhantomData;
 
-use {Decode, DecodeBuf, Encode, EncodeBuf, ErrorKind, ExactBytesEncode, Result};
+use {ByteCount, Decode, Encode, Eos, ErrorKind, ExactBytesEncode, Result};
 
 /// An object for starting a chain of decoders.
 ///
 /// # Examples
 ///
 /// ```
-/// use bytecodec::{Decode, DecodeBuf, DecodeExt, StartDecoderChain};
+/// use bytecodec::{Decode, DecodeExt, StartDecoderChain};
 /// use bytecodec::fixnum::U8Decoder;
+/// use bytecodec::io::IoDecodeExt;
 ///
 /// let mut decoder = StartDecoderChain
 ///     .chain(U8Decoder::new())
 ///     .chain(U8Decoder::new())
 ///     .chain(U8Decoder::new());
 ///
-/// let mut input = DecodeBuf::new(b"foobar");
+/// let mut input = &b"foobar"[..];
 ///
-/// let item = decoder.decode(&mut input).unwrap();
-/// assert_eq!(item, Some((b'f', b'o', b'o')));
+/// let item = decoder.decode_exact(&mut input).unwrap();
+/// assert_eq!(item, (b'f', b'o', b'o'));
 ///
-/// let item = decoder.decode(&mut input).unwrap();
-/// assert_eq!(item, Some((b'b', b'a', b'r')));
+/// let item = decoder.decode_exact(&mut input).unwrap();
+/// assert_eq!(item, (b'b', b'a', b'r'));
 /// ```
 #[derive(Debug)]
 pub struct StartDecoderChain;
@@ -48,20 +49,16 @@ where
 {
     type Item = (D::Item,);
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
-        Ok(self.0.b.decode(buf)?.map(|i| (i,)))
+    fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        self.0.b.decode(buf, eos).map(|(n, i)| (n, i.map(|i| (i,))))
     }
 
     fn has_terminated(&self) -> bool {
         self.0.b.has_terminated()
     }
 
-    fn is_idle(&self) -> bool {
-        self.0.b.is_idle()
-    }
-
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.0.b.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.0.b.requiring_bytes()
     }
 }
 impl<D0, D1, T0> Decode for DecoderChain<D0, D1, (T0,)>
@@ -71,20 +68,18 @@ where
 {
     type Item = (T0, D1::Item);
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
-        Ok(self.0.decode(buf)?.map(|(t, i)| (t.0, i)))
+    fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        self.0
+            .decode(buf, eos)
+            .map(|(n, x)| (n, x.map(|(t, i)| (t.0, i))))
     }
 
     fn has_terminated(&self) -> bool {
         self.0.has_terminated()
     }
 
-    fn is_idle(&self) -> bool {
-        self.0.is_idle()
-    }
-
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.0.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.0.requiring_bytes()
     }
 }
 impl<D0, D1, T0, T1> Decode for DecoderChain<D0, D1, (T0, T1)>
@@ -94,20 +89,18 @@ where
 {
     type Item = (T0, T1, D1::Item);
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
-        Ok(self.0.decode(buf)?.map(|(t, i)| (t.0, t.1, i)))
+    fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        self.0
+            .decode(buf, eos)
+            .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, i))))
     }
 
     fn has_terminated(&self) -> bool {
         self.0.has_terminated()
     }
 
-    fn is_idle(&self) -> bool {
-        self.0.is_idle()
-    }
-
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.0.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.0.requiring_bytes()
     }
 }
 impl<D0, D1, T0, T1, T2> Decode for DecoderChain<D0, D1, (T0, T1, T2)>
@@ -117,20 +110,18 @@ where
 {
     type Item = (T0, T1, T2, D1::Item);
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
-        Ok(self.0.decode(buf)?.map(|(t, i)| (t.0, t.1, t.2, i)))
+    fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        self.0
+            .decode(buf, eos)
+            .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, t.2, i))))
     }
 
     fn has_terminated(&self) -> bool {
         self.0.has_terminated()
     }
 
-    fn is_idle(&self) -> bool {
-        self.0.is_idle()
-    }
-
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.0.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.0.requiring_bytes()
     }
 }
 impl<D0, D1, T0, T1, T2, T3> Decode for DecoderChain<D0, D1, (T0, T1, T2, T3)>
@@ -140,20 +131,18 @@ where
 {
     type Item = (T0, T1, T2, T3, D1::Item);
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
-        Ok(self.0.decode(buf)?.map(|(t, i)| (t.0, t.1, t.2, t.3, i)))
+    fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        self.0
+            .decode(buf, eos)
+            .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, t.2, t.3, i))))
     }
 
     fn has_terminated(&self) -> bool {
         self.0.has_terminated()
     }
 
-    fn is_idle(&self) -> bool {
-        self.0.is_idle()
-    }
-
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.0.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.0.requiring_bytes()
     }
 }
 impl<D0, D1, T0, T1, T2, T3, T4> Decode for DecoderChain<D0, D1, (T0, T1, T2, T3, T4)>
@@ -163,22 +152,18 @@ where
 {
     type Item = (T0, T1, T2, T3, T4, D1::Item);
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
-        Ok(self.0
-            .decode(buf)?
-            .map(|(t, i)| (t.0, t.1, t.2, t.3, t.4, i)))
+    fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        self.0
+            .decode(buf, eos)
+            .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, t.2, t.3, t.4, i))))
     }
 
     fn has_terminated(&self) -> bool {
         self.0.has_terminated()
     }
 
-    fn is_idle(&self) -> bool {
-        self.0.is_idle()
-    }
-
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.0.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.0.requiring_bytes()
     }
 }
 impl<D0, D1, T0, T1, T2, T3, T4, T5> Decode for DecoderChain<D0, D1, (T0, T1, T2, T3, T4, T5)>
@@ -188,22 +173,18 @@ where
 {
     type Item = (T0, T1, T2, T3, T4, T5, D1::Item);
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
-        Ok(self.0
-            .decode(buf)?
-            .map(|(t, i)| (t.0, t.1, t.2, t.3, t.4, t.5, i)))
+    fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        self.0
+            .decode(buf, eos)
+            .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, t.2, t.3, t.4, t.5, i))))
     }
 
     fn has_terminated(&self) -> bool {
         self.0.has_terminated()
     }
 
-    fn is_idle(&self) -> bool {
-        self.0.is_idle()
-    }
-
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.0.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.0.requiring_bytes()
     }
 }
 impl<D0, D1, T0, T1, T2, T3, T4, T5, T6> Decode
@@ -214,22 +195,18 @@ where
 {
     type Item = (T0, T1, T2, T3, T4, T5, T6, D1::Item);
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
-        Ok(self.0
-            .decode(buf)?
-            .map(|(t, i)| (t.0, t.1, t.2, t.3, t.4, t.5, t.6, i)))
+    fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        self.0
+            .decode(buf, eos)
+            .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, t.2, t.3, t.4, t.5, t.6, i))))
     }
 
     fn has_terminated(&self) -> bool {
         self.0.has_terminated()
     }
 
-    fn is_idle(&self) -> bool {
-        self.0.is_idle()
-    }
-
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.0.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.0.requiring_bytes()
     }
 }
 
@@ -238,20 +215,19 @@ where
 /// # Examples
 ///
 /// ```
-/// use bytecodec::{Encode, EncodeBuf, EncodeExt, StartEncoderChain};
+/// use bytecodec::{Encode, EncodeExt, StartEncoderChain};
 /// use bytecodec::bytes::Utf8Encoder;
 /// use bytecodec::fixnum::U8Encoder;
+/// use bytecodec::io::IoEncodeExt;
 ///
-/// let mut output = [0; 4];
+/// let mut output = Vec::new();
 /// let mut encoder = StartEncoderChain
 ///     .chain(U8Encoder::new())
 ///     .chain(Utf8Encoder::new())
 ///     .map_from(|s: String| (s.len() as u8, s));
-/// {
-///     encoder.start_encoding("foo".to_owned()).unwrap();
-///     encoder.encode(&mut EncodeBuf::new(&mut output)).unwrap();
-/// }
-/// assert_eq!(output.as_ref(), b"\x03foo");
+/// encoder.start_encoding("foo".to_owned()).unwrap();
+/// encoder.encode_all(&mut output).unwrap();
+/// assert_eq!(output, b"\x03foo");
 /// ```
 #[derive(Debug)]
 pub struct StartEncoderChain;
@@ -264,7 +240,7 @@ impl StartEncoderChain {
 impl Encode for StartEncoderChain {
     type Item = ();
 
-    fn encode(&mut self, _buf: &mut EncodeBuf) -> Result<()> {
+    fn encode(&mut self, _buf: &mut [u8], _eos: Eos) -> Result<usize> {
         track_panic!(ErrorKind::Other)
     }
 
@@ -276,11 +252,15 @@ impl Encode for StartEncoderChain {
         true
     }
 
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        Some(0)
+    fn requiring_bytes(&self) -> ByteCount {
+        ByteCount::Finite(0)
+    }
+}
+impl ExactBytesEncode for StartEncoderChain {
+    fn exact_requiring_bytes(&self) -> u64 {
+        0
     }
 }
-impl ExactBytesEncode for StartEncoderChain {}
 
 /// Combinator for connecting encoders to a chain.
 ///
@@ -304,16 +284,16 @@ where
 {
     type Item = (E::Item,);
 
-    fn encode(&mut self, buf: &mut EncodeBuf) -> Result<()> {
-        self.inner.b.encode(buf)
+    fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
+        self.inner.b.encode(buf, eos)
     }
 
     fn start_encoding(&mut self, t: Self::Item) -> Result<()> {
         self.inner.b.start_encoding(t.0)
     }
 
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.inner.b.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.inner.b.requiring_bytes()
     }
 
     fn is_idle(&self) -> bool {
@@ -327,16 +307,16 @@ where
 {
     type Item = (T0, E1::Item);
 
-    fn encode(&mut self, buf: &mut EncodeBuf) -> Result<()> {
-        self.inner.encode(buf)
+    fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
+        self.inner.encode(buf, eos)
     }
 
     fn start_encoding(&mut self, t: Self::Item) -> Result<()> {
         self.inner.start_encoding(((t.0,), t.1))
     }
 
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.inner.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.inner.requiring_bytes()
     }
 
     fn is_idle(&self) -> bool {
@@ -350,16 +330,16 @@ where
 {
     type Item = (T0, T1, E1::Item);
 
-    fn encode(&mut self, buf: &mut EncodeBuf) -> Result<()> {
-        self.inner.encode(buf)
+    fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
+        self.inner.encode(buf, eos)
     }
 
     fn start_encoding(&mut self, t: Self::Item) -> Result<()> {
         self.inner.start_encoding(((t.0, t.1), t.2))
     }
 
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.inner.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.inner.requiring_bytes()
     }
 
     fn is_idle(&self) -> bool {
@@ -373,16 +353,16 @@ where
 {
     type Item = (T0, T1, T2, E1::Item);
 
-    fn encode(&mut self, buf: &mut EncodeBuf) -> Result<()> {
-        self.inner.encode(buf)
+    fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
+        self.inner.encode(buf, eos)
     }
 
     fn start_encoding(&mut self, t: Self::Item) -> Result<()> {
         self.inner.start_encoding(((t.0, t.1, t.2), t.3))
     }
 
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.inner.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.inner.requiring_bytes()
     }
 
     fn is_idle(&self) -> bool {
@@ -396,16 +376,16 @@ where
 {
     type Item = (T0, T1, T2, T3, E1::Item);
 
-    fn encode(&mut self, buf: &mut EncodeBuf) -> Result<()> {
-        self.inner.encode(buf)
+    fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
+        self.inner.encode(buf, eos)
     }
 
     fn start_encoding(&mut self, t: Self::Item) -> Result<()> {
         self.inner.start_encoding(((t.0, t.1, t.2, t.3), t.4))
     }
 
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.inner.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.inner.requiring_bytes()
     }
 
     fn is_idle(&self) -> bool {
@@ -419,16 +399,16 @@ where
 {
     type Item = (T0, T1, T2, T3, T4, E1::Item);
 
-    fn encode(&mut self, buf: &mut EncodeBuf) -> Result<()> {
-        self.inner.encode(buf)
+    fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
+        self.inner.encode(buf, eos)
     }
 
     fn start_encoding(&mut self, t: Self::Item) -> Result<()> {
         self.inner.start_encoding(((t.0, t.1, t.2, t.3, t.4), t.5))
     }
 
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.inner.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.inner.requiring_bytes()
     }
 
     fn is_idle(&self) -> bool {
@@ -442,8 +422,8 @@ where
 {
     type Item = (T0, T1, T2, T3, T4, T5, E1::Item);
 
-    fn encode(&mut self, buf: &mut EncodeBuf) -> Result<()> {
-        self.inner.encode(buf)
+    fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
+        self.inner.encode(buf, eos)
     }
 
     fn start_encoding(&mut self, t: Self::Item) -> Result<()> {
@@ -451,8 +431,8 @@ where
             .start_encoding(((t.0, t.1, t.2, t.3, t.4, t.5), t.6))
     }
 
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.inner.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.inner.requiring_bytes()
     }
 
     fn is_idle(&self) -> bool {
@@ -467,8 +447,8 @@ where
 {
     type Item = (T0, T1, T2, T3, T4, T5, T6, E1::Item);
 
-    fn encode(&mut self, buf: &mut EncodeBuf) -> Result<()> {
-        self.inner.encode(buf)
+    fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
+        self.inner.encode(buf, eos)
     }
 
     fn start_encoding(&mut self, t: Self::Item) -> Result<()> {
@@ -476,8 +456,8 @@ where
             .start_encoding(((t.0, t.1, t.2, t.3, t.4, t.5, t.6), t.7))
     }
 
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.inner.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.inner.requiring_bytes()
     }
 
     fn is_idle(&self) -> bool {
@@ -490,8 +470,8 @@ where
     E0: ExactBytesEncode,
     E1: ExactBytesEncode,
 {
-    fn requiring_bytes(&self) -> u64 {
-        self.inner.requiring_bytes()
+    fn exact_requiring_bytes(&self) -> u64 {
+        self.inner.exact_requiring_bytes()
     }
 }
 
@@ -512,19 +492,22 @@ where
 {
     type Item = (A::Item, B::Item);
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
+    fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        let mut offset = 0;
         if self.a.item.is_none() {
-            track!(self.a.decode(buf))?;
+            let (size, _always_none) = track!(self.a.decode(buf, eos))?;
             if self.a.item.is_none() {
-                return Ok(None);
+                return Ok((size, None));
             }
+            offset = size;
         }
-        if let Some(b) = track!(self.b.decode(buf))? {
+
+        let (size, item) = track!(self.b.decode(&buf[offset..], eos))?;
+        let item = item.map(|b| {
             let a = self.a.item.take().expect("Never fails");
-            Ok(Some((a, b)))
-        } else {
-            Ok(None)
-        }
+            (a, b)
+        });
+        Ok((offset + size, item))
     }
 
     fn has_terminated(&self) -> bool {
@@ -535,22 +518,15 @@ where
         }
     }
 
-    fn is_idle(&self) -> bool {
-        self.a.item.is_none() && self.a.is_idle()
-    }
-
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        let a = self.a
-            .item
-            .as_ref()
-            .map(|_| 0)
-            .or_else(|| self.a.requiring_bytes_hint());
-        let b = self.b.requiring_bytes_hint();
+    fn requiring_bytes(&self) -> ByteCount {
+        let a = self.a.requiring_bytes();
+        let b = self.b.requiring_bytes();
         match (a, b) {
-            (Some(a), Some(b)) => Some(a + b),
-            (Some(a), None) => Some(a),
-            (None, Some(b)) => Some(b),
-            (None, None) => None,
+            (ByteCount::Finite(a), ByteCount::Finite(b)) => ByteCount::Finite(a + b),
+            (ByteCount::Infinite, _) | (_, ByteCount::Infinite) => ByteCount::Infinite,
+            (ByteCount::Finite(a), _) => ByteCount::Finite(a),
+            (_, ByteCount::Finite(b)) => ByteCount::Finite(b),
+            (ByteCount::Unknown, ByteCount::Unknown) => ByteCount::Unknown,
         }
     }
 }
@@ -561,15 +537,12 @@ where
 {
     type Item = (A::Item, B::Item);
 
-    fn encode(&mut self, buf: &mut EncodeBuf) -> Result<()> {
-        while !buf.is_empty() && !self.is_idle() {
-            if !self.a.is_idle() {
-                track!(self.a.encode(buf))?;
-            } else {
-                track!(self.b.encode(buf))?;
-            }
+    fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
+        if !self.a.is_idle() {
+            track!(self.a.encode(buf, eos))
+        } else {
+            track!(self.b.encode(buf, eos))
         }
-        Ok(())
     }
 
     fn start_encoding(&mut self, item: Self::Item) -> Result<()> {
@@ -580,10 +553,14 @@ where
         Ok(())
     }
 
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.a
-            .requiring_bytes_hint()
-            .and_then(|a| self.b.requiring_bytes_hint().map(|b| a + b))
+    fn requiring_bytes(&self) -> ByteCount {
+        let a = self.a.requiring_bytes();
+        let b = self.b.requiring_bytes();
+        match (a, b) {
+            (ByteCount::Finite(a), ByteCount::Finite(b)) => ByteCount::Finite(a + b),
+            (ByteCount::Infinite, _) | (_, ByteCount::Infinite) => ByteCount::Infinite,
+            (ByteCount::Unknown, _) | (_, ByteCount::Unknown) => ByteCount::Unknown,
+        }
     }
 
     fn is_idle(&self) -> bool {
@@ -595,8 +572,8 @@ where
     E0: ExactBytesEncode,
     E1: ExactBytesEncode,
 {
-    fn requiring_bytes(&self) -> u64 {
-        self.a.requiring_bytes() + self.b.requiring_bytes()
+    fn exact_requiring_bytes(&self) -> u64 {
+        self.a.exact_requiring_bytes() + self.b.exact_requiring_bytes()
     }
 }
 
@@ -616,38 +593,38 @@ impl<T, I> Buffered<T, I> {
 impl<T: Decode> Decode for Buffered<T, T::Item> {
     type Item = T::Item;
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
+    fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
         if self.item.is_none() {
-            self.item = track!(self.decoder.decode(buf))?;
+            let (size, item) = track!(self.decoder.decode(buf, eos))?;
+            self.item = item;
+            Ok((size, None))
+        } else {
+            Ok((0, None))
         }
-        Ok(None)
     }
 
     fn has_terminated(&self) -> bool {
         if self.item.is_some() {
-            true
+            false
         } else {
             self.decoder.has_terminated()
         }
     }
 
-    fn is_idle(&self) -> bool {
-        self.item.is_some() || self.decoder.is_idle()
-    }
-
-    fn requiring_bytes_hint(&self) -> Option<u64> {
+    fn requiring_bytes(&self) -> ByteCount {
         if self.item.is_some() {
-            Some(0)
+            ByteCount::Finite(0)
         } else {
-            self.decoder.requiring_bytes_hint()
+            self.decoder.requiring_bytes()
         }
     }
 }
 
 #[cfg(test)]
 mod test {
-    use {Decode, DecodeBuf, DecodeExt, StartDecoderChain};
+    use {DecodeExt, StartDecoderChain};
     use fixnum::U8Decoder;
+    use io::IoDecodeExt;
 
     #[test]
     fn it_works() {
@@ -657,8 +634,8 @@ mod test {
             .chain(U8Decoder::new());
 
         assert_eq!(
-            track_try_unwrap!(decoder.decode(&mut DecodeBuf::new(b"foo"))),
-            Some((b'f', b'o', b'o'))
+            track_try_unwrap!(decoder.decode_exact(b"foo".as_ref())),
+            (b'f', b'o', b'o')
         );
     }
 }
