@@ -7,7 +7,7 @@ use serde::{Deserialize, Serialize};
 use serde_json;
 use trackable::error::ErrorKindExt;
 
-use {Decode, DecodeBuf, Encode, EncodeBuf, ErrorKind, Result};
+use {ByteCount, Decode, Encode, Eos, ErrorKind, Result};
 use monolithic::{MonolithicDecode, MonolithicDecoder, MonolithicEncode, MonolithicEncoder};
 
 /// JSON decoder.
@@ -20,18 +20,16 @@ use monolithic::{MonolithicDecode, MonolithicDecoder, MonolithicEncode, Monolith
 /// ```
 /// # extern crate bytecodec;
 /// # extern crate serde_json;
-/// use bytecodec::{Decode, DecodeBuf};
+/// use bytecodec::{Decode, Eos};
 /// use bytecodec::json_codec::JsonDecoder;
 /// use serde_json::Value;
 ///
 /// # fn main() {
 /// let mut decoder = JsonDecoder::<Value>::new();
 ///
-/// decoder.decode(&mut DecodeBuf::new(b"[1, 2")).unwrap();
-/// let json = decoder
-///    .decode(&mut DecodeBuf::with_eos(b", 3]", true))
-///    .unwrap()
-///    .unwrap();
+/// decoder.decode(b"[1, 2", Eos::new(false)).unwrap();
+/// let (_, item) = decoder.decode(b", 3]", Eos::new(true)).unwrap();
+/// let json = item.unwrap();
 ///
 /// assert_eq!(json.to_string(), "[1,2,3]");
 /// # }
@@ -53,20 +51,16 @@ where
 {
     type Item = T;
 
-    fn decode(&mut self, buf: &mut DecodeBuf) -> Result<Option<Self::Item>> {
-        track!(self.0.decode(buf))
+    fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        track!(self.0.decode(buf, eos))
     }
 
     fn has_terminated(&self) -> bool {
         self.0.has_terminated()
     }
 
-    fn is_idle(&self) -> bool {
-        self.0.is_idle()
-    }
-
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.0.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.0.requiring_bytes()
     }
 }
 impl<T> Default for JsonDecoder<T>
@@ -117,8 +111,8 @@ where
 {
     type Item = T;
 
-    fn encode(&mut self, buf: &mut EncodeBuf) -> Result<()> {
-        track!(self.0.encode(buf))
+    fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
+        track!(self.0.encode(buf, eos))
     }
 
     fn start_encoding(&mut self, item: Self::Item) -> Result<()> {
@@ -129,8 +123,8 @@ where
         self.0.is_idle()
     }
 
-    fn requiring_bytes_hint(&self) -> Option<u64> {
-        self.0.requiring_bytes_hint()
+    fn requiring_bytes(&self) -> ByteCount {
+        self.0.requiring_bytes()
     }
 }
 impl<T> Default for JsonEncoder<T>
@@ -167,18 +161,16 @@ where
 mod test {
     use serde_json::Value;
 
-    use {Decode, DecodeBuf};
+    use {Decode, Eos};
     use json_codec::JsonDecoder;
 
     #[test]
     fn json_decoder_works() {
         let mut decoder = JsonDecoder::<Value>::new();
 
-        decoder.decode(&mut DecodeBuf::new(b"[1, 2")).unwrap();
-        let json = decoder
-            .decode(&mut DecodeBuf::with_eos(b", 3]", true))
-            .unwrap()
-            .unwrap();
+        decoder.decode(b"[1, 2", Eos::new(false)).unwrap();
+        let (_, item) = decoder.decode(b", 3]", Eos::new(true)).unwrap();
+        let json = item.unwrap();
 
         assert_eq!(json.to_string(), "[1,2,3]");
     }
